﻿using System;
using System.Collections.Generic;
//using System.Linq;
using System.Text;
using Ivi.Visa.Interop;
//'' """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//''  © Agilent Technologies, Inc. 2013
//''
//'' You have a royalty-free right to use, modify, reproduce and distribute
//'' the Sample Application Files (and/or any modified version) in any way
//'' you find useful, provided that you agree that Agilent Technologies has no
//'' warranty,  obligations or liability for any Sample Application Files.
//''
//'' Agilent Technologies provides programming examples for illustration only,
//'' This sample program assumes that you are familiar with the programming
//'' language being demonstrated and the tools used to create and debug
//'' procedures. Agilent Technologies support engineers can help explain the
//'' functionality of Agilent Technologies software components and associated
//'' commands, but they will not modify these samples to provide added
//'' functionality or construct procedures to meet your specific needs.
//'' """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

namespace DMMScreenCapture
{
    class Histogram
    {
        static void Main(string[] args)
        {
            Histogram DmmClass = new Histogram(); //Create an instance of this class so we can call functions from Main

            //Programmatically sets the display mode to histogram views. 
            // Takes a sample of 500 readings per trigger and reports on the results while the display shows the distribution in histogram mode. 
            //Then downloads a screen capture to the PC. Screen capture illustrates IEEE block transfer method.

            //For more information on getting started using VISA COM I/O operations see the app note located at:
            //http://cp.literature.agilent.com/litweb/pdf/5989-6338EN.pdf

            Ivi.Visa.Interop.ResourceManager rm = new Ivi.Visa.Interop.ResourceManager(); //Open up a new resource manager
            Ivi.Visa.Interop.FormattedIO488 myDmm = new Ivi.Visa.Interop.FormattedIO488(); //Open a new Formatted IO 488 session 
            try
            {
                //string DutAddr = "GPIB0::22"; //String for GPIB
                //string DutAddr = ""TCPIP0::169.254.4.61";  //Example string for LAN
                //string DutAddr = "USB0::0x0957::0x1A07::MY53000101"; //Example string for USB
                string DutAddr = "USB0::0x0957::0x1C07::US00000069::0::INSTR"; //String for USB

                myDmm.IO = (IMessage)rm.Open(DutAddr, AccessMode.NO_LOCK, 2000, ""); //Open up a handle to the DMM with a 2 second timeout
                myDmm.IO.Timeout = 10000; //You can also set your timeout by doing this command, sets Timeout to 10 seconds

                //First start with a reset state
                myDmm.IO.Clear(); //Send a device clear first to stop any measurements in process
                myDmm.WriteString("*RST", true); //Reset the device
                myDmm.WriteString("*IDN?", true); //Get the IDN string                
                string IDN = myDmm.ReadString();
                Console.WriteLine(IDN); //report the DMM's identity

                //Set the Display to Histogram mode, DCV and get 100 readings
                
                //Configure for DCV 100V range, 10mV resolution this is a fast reading
                myDmm.WriteString("CONF:VOLT:DC 100, 0.01", true);

                myDmm.WriteString("DISP:VIEW HIST", true); //Set to HISTogram view, options are NUMeric|HISTogram|TCHart|METer

                myDmm.WriteString("SAMP:COUN 500", true); //Get 500 Readings per trigger
                myDmm.WriteString("READ?", true); //Trigger the DMM to start the readings, can also use INIT command
                Object[] DCVResult = (object[])myDmm.ReadList(IEEEASCIIType.ASCIIType_Any, ",;"); //Get the readings in an array                
                foreach (double rdg in DCVResult) //Write out each result to the console
                    Console.WriteLine(rdg.ToString());
                
                DmmClass.CheckDMMError(myDmm); //Check if the DMM has any errors

                //Here's where you can get a screen capture of the display
                myDmm.WriteString("HCOP:SDUM:DATA:FORM PNG", true); //Set the format to PNG or BMP
                myDmm.WriteString("HCOP:SDUM:DATA?", true); //Send the command to get the screen data
                byte[] junk = (byte[])myDmm.ReadIEEEBlock(IEEEBinaryType.BinaryType_UI1,false,true); //Read the data back from DMM. DMM returns in IEEE block format
                DmmClass.CheckDMMError(myDmm); //Check DMM for Errors

                string OutFilename = "C:\\Temp\\ScreenCapture.png"; //Set picture location
                System.IO.FileStream f = System.IO.File.Open(OutFilename, System.IO.FileMode.Create); //Create/Overwrite the local copy of the file

                f.Write(junk, 0, junk.Length); //Write the picture data
                f.Close(); //Close picture file
                Console.WriteLine("DMM Display saved at: " + OutFilename);
                
            }
            catch (Exception e)
            {
                Console.WriteLine("Error occured: " + e.Message);
            }
            finally
            {
                //Close out your resources
                try { myDmm.IO.Close(); }
                catch { }
                try { System.Runtime.InteropServices.Marshal.ReleaseComObject(myDmm); }
                catch { }
                try
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(rm);
                }
                catch { }
                Console.WriteLine("Press any key to continue...");
                Console.ReadKey();

            }
        }
        public void CheckDMMError(FormattedIO488 myDmm)
        {

            myDmm.WriteString("SYST:ERR?", true);
            string errStr = myDmm.ReadString();

            if (errStr.Contains("No error")) //If no error, then return
                return;
            //If there is an error, read out all of the errors and return them in an exception
            else
            {
                string errStr2 = "";
                do
                {
                    myDmm.WriteString("SYST:ERR?", true);
                    errStr2 = myDmm.ReadString();
                    if (!errStr2.Contains("No error")) errStr = errStr + "\n" + errStr2;

                } while (!errStr2.Contains("No error"));
                throw new Exception("Exception: Encountered system error(s)\n" + errStr);
            }

        }
    }
}

